#!/usr/bin/env perl
use strict;
use warnings;

use Path::Class qw/file/;
use constant UCHAR_MAX => 256;
use POSIX qw/ceil/;

my $path = shift or die "usage: %0 <file>";
my $fh = file($path)->openr;

my @count;
for (my $i = 0; $i < UCHAR_MAX; $i++) {
    $count[$i] = 0;
}

my $n = 0;
while (defined (my $c = $fh->getc)) {
    $count[ord $c]++;
    $n++;
}

my $entropy = 0;
for my $v (@count) {
    next if $v == 0;
    my $p = $v / $n;
    $entropy -= $p * log($p);
}

$entropy = $entropy / log(2);
my $low  = ceil($n / 8 * $entropy);

printf "size: %d, entropy: %.2f, lower bound: %d\n", $n, $entropy, $low;
