#!/usr/bin/env perl
use strict;
use warnings;

use FindBin::libs;
use Perl6::Say;
use IO::String;
use HuffmanCoder;

my $text = shift or die "usage: %0 <text>";
my @buff = unpack('C*', $text);

my @count;
for (my $i = 0; $i < 256; $i++) {
    $count[$i] = 0;
}

for my $c (@buff) {
    $count[$c]++;
}

my $huff = HuffmanCoder->new(\@count);
$huff->show_code;

my $in  = IO::String->new($text);
my $out = IO::String->new;

$huff->encode($in, $out, length $text);

$out->seek(0, 0);
my $bin;
while ($out->read(my $buff, 1024)) {
    $bin .= $buff;
}

say sprintf "%d bytes => %d bytes", length($text), length($bin);

my $res = IO::String->new;
HuffmanCoder->decode(IO::String->new($bin), $res);
$res->seek(0, 0);
say $res->getline;
