#!/usr/bin/env perl
use strict;
use warnings;

use FindBin::libs;
use Perl6::Say;
use IO::String;
use Benchmark::Timer;

use HuffmanCoder;

use Path::Class qw/file/;

my $path = shift or die "usage: %0 <file>";
my $file = file($path);
my $in = $file->openr;

my @count;
for (my $i = 0; $i < 256; $i++) {
    $count[$i] = 0;
}

while (defined(my $c = $in->getc)) {
    $count[ord $c]++;
}

$in->seek(0, 0);
my $out = IO::String->new;

my $t = Benchmark::Timer->new;
$t->start('huff_encode');
my $huff = HuffmanCoder->new(\@count);
# $huff->show_code;
$huff->encode($in, $out, $file->stat->size);
$t->stop('huff_encode');

$out->seek(0, 0);
my $bin;
while ($out->read(my $buff, 1024)) {
    $bin .= $buff;
}

warn $file->stat->size;
warn length $bin;
warn scalar $t->reports;

print $bin;
